# _dev_typeesm.time()
# _dev_typeesm.etime()
# _dev_typeesm.status()
# _dev_typeesm.disks()
# _dev_typeesm.ram()
# _dev_typeesm.backup_status()
# _dev_typeesm.callhome()
# _dev_typeesm.rules_status()
# _dev_typeesm._get_timezones()
# _dev_typeesm.tz_offsets()
# _dev_typeesm.timezones()
# _dev_typeesm.tz_name_to_id('Darwin')
# _dev_typeesm.tz_id_to_name('1')
# _dev_typeesm.type_id_to_venmod('65')
# _dev_typeesm.venmod_to_type_id('UNIX', 'Linux')
# _dev_typeesm._get_ds_types()
# _dev_typeesm.recs()
